/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.component;

import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;

import org.ow2.petals.jbi.messaging.PetalsDeliveryChannel;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.microkernel.api.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.exception.RoutingException;
import org.ow2.petals.microkernel.api.jbi.security.SecurityContext;

/**
 * The PEtALS component context interface.
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 */
public interface PetalsComponentContext extends ComponentContext {

    SecurityContext getSecurityContext();

    void setSecurityContext(SecurityContext securityContext);

    /**
     * deregister all endpoints (internal and external)
     * 
     * Used by the {@link DeliveryChannel} when closing
     * 
     */
    void deactivateAllEndpoints();

    /**
     * Used by the {@link DeliveryChannel} to send message into the router
     */
    RouterService getRouter();

    ContainerConfiguration getContainerConfiguration();

    Component getComponent();

    void setComponent(Component component);

    /**
     * Clear the component Context, remove objects dependent to the component life cycle scope. To be used when to
     * component is uninstalling.
     */
    void clear();

    /**
     * Do not create a delivery channel if none are present or if closed
     */
    PetalsDeliveryChannel getDeliveryChannelInstance();

    /**
     * Called by the router to notify a component there is an exchange for it.
     * 
     * At that point the router has reverted the role to its correct value w.r.t. the component and called the necessary
     * ReceiverModule.
     */
    void offerExchange(PetalsMessageExchange exchange) throws RoutingException;

    /**
     * Used by the router to know if the component still has exchanges in its queue.
     */
    boolean hasExchangesInDeliveryChannel();

}
